package as.adamsmith.etherealdialpad;

import java.util.*;

import android.content.*;
import android.graphics.*;
import android.os.Bundle;
import android.os.RemoteException;
import android.util.FloatMath;
import android.view.*;

public class TrailPad extends Pad {

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(new TrailView(this));
	}
	
	class TrailView extends View {

		final List<Float> xHistory;
		final List<Float> yHistory;
		final Path path = new Path();
		final Paint pathPaint = new Paint();
		final Paint fingerPaint = new Paint();
		
		public int manualOffset;
		
		long lastTick;
		float x, y;
		boolean isTouching;
		
		float lastX;
		float lastY;
		float d;
		final float maxD = 512f;
		
		public TrailView(Context context) {
			super(context);
			
			xHistory = new ArrayList<Float>();
			yHistory = new ArrayList<Float>();
			
			setBackgroundColor(0xFF000000);
			
			pathPaint.setARGB(255, 255,255,255);
			pathPaint.setStyle(Paint.Style.STROKE);
			pathPaint.setStrokeWidth(4);
			//pathPaint.setAntiAlias(true);
			//pathPaint.setStrokeCap(Paint.Cap.ROUND);
			pathPaint.setShadowLayer(6, 0,0, 0xFFFFFFFF);
			
			fingerPaint.setARGB(128, 255,255,255);
			fingerPaint.setStyle(Paint.Style.FILL);
			//fingerPaint.setAntiAlias(true);
			fingerPaint.setShadowLayer(10, 0, 0, 0xFFFFFFFF);
			
			setKeepScreenOn(true);
			setFocusable(true);
		}
		
		@Override
	    public boolean onTouchEvent(MotionEvent event) {
			float ex = event.getX();
			float ey = event.getY();
			
			x = ex / getWidth();
			y = 1 - ey / getHeight();
			    
	    	if(event.getAction() == MotionEvent.ACTION_MOVE) {
	    		try {
					synthService.primaryXY(x, y);
				} catch (RemoteException e) {
					e.printStackTrace();
				}
	    		
	    		xHistory.add(ex);
	    		yHistory.add(ey);
	    		d += FloatMath.sqrt((ex-lastX)*(ex-lastX)+(ey-lastY)*(ey-lastY));
	    		while(d > maxD && xHistory.size() > 1) {
	    			float surplus = d - maxD;
	    			float ax = xHistory.get(0);
	    			float ay = yHistory.get(0);
	    			float bx = xHistory.get(1);
	    			float by = yHistory.get(1);
	    			float chopD = FloatMath.sqrt((ax-bx)*(ax-bx)+(ay-by)*(ay-by));
	    			if(surplus > chopD) {
	    				xHistory.remove(0);
	    				yHistory.remove(0);
	    				d -= chopD;
	    			} else {
	    				float frac = surplus / chopD;
	    				float nx = (1-frac)*ax + frac*bx;
	    				float ny = (1-frac)*ay + frac*by;
	    				xHistory.set(0, nx);
	    				yHistory.set(0, ny);
	    				d -= surplus;
	    				break;
	    			}
	    		}
	    	}
	    	
	    	if(event.getAction() == MotionEvent.ACTION_DOWN) {
	    		isTouching = true;
	    		try {
					synthService.primaryOn();
					synthService.primaryXY(x, y);
				} catch (RemoteException e) {
					e.printStackTrace();
				}
	    		xHistory.clear();
	    		yHistory.clear();
	    		xHistory.add(ex);
	    		yHistory.add(ey);
	    		d = 0;
	    	}
	    	
	    	if(event.getAction() == MotionEvent.ACTION_UP) {
	    		isTouching = false;
	    		try {
					synthService.primaryOff();
					synthService.primaryXY(x, y);
				} catch (RemoteException e) {
					e.printStackTrace();
				}
	    		xHistory.clear();
	    		yHistory.clear();
	    		d = 0;
	    	}
	    	
			lastX = ex;
			lastY = ey;
	    	
	    	invalidate();
	    	
	    	return true;
	    }
		
		@Override
		public void onDraw(Canvas canvas) {
			
			path.rewind();
			if(xHistory.size() > 0) {
				path.moveTo(xHistory.get(0), yHistory.get(0));
			}
			int steps = xHistory.size() - 1;
			for(int i = 0; i < steps; i++) {
				float xA = xHistory.get(i);
				float yA = yHistory.get(i);
				float xB = xHistory.get(i+1);
				float yB = yHistory.get(i+1);
				float xP = (xA+xB)/2;
				float yP = (yA+yB)/2;
				path.quadTo(xA, yA,
						    xP, yP);
			}
			if(xHistory.size() > 0) {
				path.lineTo(xHistory.get(steps), yHistory.get(steps));
			}
			
			
			canvas.drawPath(path, pathPaint);
			if(isTouching) {
				canvas.drawCircle(lastX, lastY, 15, fingerPaint);
			}
		}
	}

}