package as.adamsmith.etherealdialpad.pads.gravitone;

import java.nio.*;
import java.util.*;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import android.app.Activity;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.opengl.GLSurfaceView;
import android.opengl.GLU;
import android.os.Bundle;
import android.os.IBinder;
import android.os.SystemClock;
import android.util.FloatMath;
import android.util.Log;
import android.view.MotionEvent;
import android.view.Window;
import android.widget.Toast;

import as.adamsmith.etherealdialpad.dsp.ISynthService;

public class GravitonePad extends Activity implements SensorEventListener {
	
	float gx, gy, gz;
	boolean isDown;
	
	GLSurfaceView view;
	Renderer renderer;
	
	SensorManager sensorManager;
	Sensor accelerometer;
	
	ISynthService synthService = null;
	SynthServiceConnection synthServiceConnection = new SynthServiceConnection();
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		getWindow().requestFeature(Window.FEATURE_NO_TITLE);
		
		renderer = new Renderer();
		view = new GLSurfaceView(this);
		view.setRenderer(renderer);
		view.setRenderMode(GLSurfaceView.RENDERMODE_WHEN_DIRTY);
		view.setDebugFlags(GLSurfaceView.DEBUG_CHECK_GL_ERROR);
		setContentView(view);
		
		sensorManager = (SensorManager)getSystemService(Context.SENSOR_SERVICE);
		
		List<Sensor> sensors = sensorManager.getSensorList(Sensor.TYPE_ACCELEROMETER);
		if(sensors.size() == 0) {
			Toast.makeText(this, R.string.no_sensor, Toast.LENGTH_LONG);
			finish();
		} else {
			for(int i = 0; i < sensors.size(); i++) {
				Log.d("gravitone", "sensor_"+i+": "+sensors.get(i).getName());
			}
			accelerometer = sensors.get(0);
		}
	}
	
	
	@Override
	public void onResume() {
		super.onResume();
		
		view.onResume();
		
		bindService(
				new Intent(ISynthService.class.getName()),
				synthServiceConnection,
				Context.BIND_AUTO_CREATE);
		
		sensorManager.registerListener(this, accelerometer, SensorManager.SENSOR_DELAY_GAME);
	}
	
	@Override
	public void onPause() {
		super.onPause();
		
		view.onPause();
		
		unbindService(synthServiceConnection);
		
		sensorManager.unregisterListener(this);
	}
	
	class SynthServiceConnection implements ServiceConnection {

		@Override
		public void onServiceConnected(ComponentName className, IBinder service) {
			synthService = ISynthService.Stub.asInterface(service);
			}

		@Override
		public void onServiceDisconnected(ComponentName className) {
			synthService = null;
		}
	}

	@Override
	public void onAccuracyChanged(Sensor sensor, int accuracy) {
		// pass
	}


	@Override
	public void onSensorChanged(SensorEvent event) {
		
		gx = event.values[0];
		gy = event.values[1];
		gz = event.values[2];
		
		try {
			synthService.primaryXY(clamp01(-gx/20.0f+0.5f), clamp01(1.0f-Math.abs(gz)/10.0f));
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		view.requestRender();
	}
	
	public float clamp01(float x) {
		return x>1f?1f:x<0?0:x;
	}
    
	@Override
	public boolean onTouchEvent(MotionEvent event) {
		if(event.getAction() == MotionEvent.ACTION_DOWN) {
			isDown = true;
			view.requestRender();
			try {
				synthService.primaryOn();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		if(event.getAction() == MotionEvent.ACTION_UP) {
			isDown = false;
			view.requestRender();
			try {
				synthService.primaryOff();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		return true;
	}
	
	
	class Renderer implements GLSurfaceView.Renderer {
		IntBuffer ringBuffer;
		
		public Renderer() {
			final int N = 32;
			final int ONE = 0x10000;
			ringBuffer = ByteBuffer.allocateDirect(4*2*(N+2)).order(ByteOrder.nativeOrder()).asIntBuffer();
			ringBuffer.put(0, 0);
			ringBuffer.put(1, 0);
			for(int i = 0; i < N; i++) {
				float t = (float)(2*Math.PI*i/N);
				ringBuffer.put(2*i+2, (int)(ONE*FloatMath.sin(t)));
				ringBuffer.put(2*i+3, (int)(ONE*FloatMath.cos(t)));
			}
			ringBuffer.put(2*N+2, 0);
			ringBuffer.put(2*N+3, ONE);
		}

		@Override
		public void onDrawFrame(GL10 gl) {
			gl.glClearColor(0f,0f,0f, 1.0f);
			gl.glClear(GL10.GL_COLOR_BUFFER_BIT|GL10.GL_DEPTH_BUFFER_BIT);
			
			gl.glLoadIdentity();
			
			float rot = (float)(-180f*Math.atan2(gx,gy)/Math.PI);
			gl.glRotatef(rot, 0, 0, 1);
			GLU.gluLookAt(gl, gx,gy,gz, 0,0,0, 0,0,10);
			
			gl.glEnableClientState(GL10.GL_VERTEX_ARRAY);
			gl.glVertexPointer(2, GL10.GL_FIXED, 0, ringBuffer);
			
			
			if(isDown) {
				gl.glColor4x(0x7000, 0x10000, 0x7000, 0x10000);
			} else {
				gl.glColor4x(0x7000, 0x7000, 0x7000, 0x10000);
			}
			gl.glDrawArrays(GL10.GL_TRIANGLE_FAN, 0, ringBuffer.capacity()/2);
			
		}

		@Override
		public void onSurfaceChanged(GL10 gl, int width, int height) {
			gl.glViewport(0, 0, width, height);
			
			gl.glMatrixMode(GL10.GL_PROJECTION);
			gl.glLoadIdentity();
			GLU.gluPerspective(gl, 45f, (float)width/height, 1f, 20f);
			
			gl.glMatrixMode(GL10.GL_MODELVIEW);
			
			//gl.glEnable(GL10.GL_DEPTH_TEST);
		}

		@Override
		public void onSurfaceCreated(GL10 gl, EGLConfig config) {
			// pass
			
		}
	}
}