package as.adamsmith.etherealdialpad.pads.nightsky;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.IntBuffer;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import android.app.Activity;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.opengl.GLSurfaceView;
import android.os.Bundle;
import android.os.IBinder;
import android.os.RemoteException;
import android.os.SystemClock;
import android.util.Log;
import android.view.MotionEvent;
import android.view.Window;

import as.adamsmith.etherealdialpad.dsp.ISynthService;

public class NightSkyPad extends Activity {
	SkyView skyView;
	ISynthService synthService = null;
	SynthServiceConnection synthServiceConnection = new SynthServiceConnection();
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		getWindow().requestFeature(Window.FEATURE_NO_TITLE);
		skyView = new SkyView(this);
        setContentView(skyView);
	}
	
	
	@Override
	public void onResume() {
		super.onResume();
		
		bindService(
				new Intent(ISynthService.class.getName()),
				synthServiceConnection,
				Context.BIND_AUTO_CREATE);
		
    	skyView.onResume();
    	skyView.requestRender();
	}
	
	@Override
	public void onPause() {
		super.onPause();
		
		unbindService(synthServiceConnection);
    	skyView.onPause();
	}
	
	class SynthServiceConnection implements ServiceConnection {

		@Override
		public void onServiceConnected(ComponentName className, IBinder service) {
			synthService = ISynthService.Stub.asInterface(service);
			}

		@Override
		public void onServiceDisconnected(ComponentName className) {
			synthService = null;
		}
	}
    
    class SkyView extends GLSurfaceView {
    	SkyRenderer skyRenderer;

    	public SkyView(Context context) {
    		super(context);
    		
    		setKeepScreenOn(true);
    		
    		skyRenderer = new SkyRenderer();
    		
    		//setDebugFlags(DEBUG_CHECK_GL_ERROR);// | DEBUG_LOG_GL_CALLS);
    		setRenderer(skyRenderer);
    		//setRenderMode(RENDERMODE_WHEN_DIRTY);
    	}

    	
    	boolean op1, op2;
    	boolean p1, p2;
    	float x0, y0, x1, x2, y1, y2;
    	
    	@Override
    	public boolean onTouchEvent(MotionEvent event) {
    		int action = event.getAction();
    		
    		
    		// NEW JUNK
    		
    		if(action == MotionEvent.ACTION_POINTER_1_DOWN) p2 = true;
    		if(action == MotionEvent.ACTION_POINTER_2_DOWN) p2 = true;
    		if(action == MotionEvent.ACTION_POINTER_3_DOWN) p2 = true;
    		if(action == MotionEvent.ACTION_POINTER_1_UP) p2 = false;
    		if(action == MotionEvent.ACTION_POINTER_2_UP) p2 = false;
    		if(action == MotionEvent.ACTION_POINTER_3_UP) p2 = false;
    		
    		if(action == MotionEvent.ACTION_POINTER_DOWN) p2 = true;
    		if(action == MotionEvent.ACTION_POINTER_UP) p2 = false;
    		
    		if(action == MotionEvent.ACTION_DOWN) p1 = true;
    		if(action == MotionEvent.ACTION_UP) p1 = false;  		
    		
    		if(p1) {
    			if(p2) {
    				x1 = event.getX(event.getPointerId(1)) / getWidth();
    				y1 = (getHeight() - event.getY(event.getPointerId(1))) / getHeight();
    			} else {
    				x1 = event.getX() / getWidth();
    				y1 = (getHeight() - event.getY()) / getHeight();
    			}
				try {
					synthService.primaryXY(x1,y1);
				} catch (RemoteException e) {
					e.printStackTrace();
				}
    		}
    		
    		if(p2) {
    			x2 = event.getX(event.getPointerId(2)) / getWidth();
    			y2 = (getHeight() - event.getY(event.getPointerId(2))) / getHeight();
				try {
					synthService.secondaryXY(x2,y2);
				} catch (RemoteException e) {
					e.printStackTrace();
				}
    		}
    		
    		if(p1 && !op1) {
    			try {
					synthService.primaryOn();
				} catch (RemoteException e) {
					e.printStackTrace();
				}
    		}
    		if(!p1 && op1) {
    			try {
					synthService.primaryOff();
				} catch (RemoteException e) {
					e.printStackTrace();
				}
    		}
    		op1 = p1;
    		
    		if(p2 && !op2) {
    			try {
					synthService.secondaryOn();
				} catch (RemoteException e) {
					e.printStackTrace();
				}
    		}
    		if(!p2 && op2) {
    			try {
					synthService.secondaryOff();
				} catch (RemoteException e) {
					e.printStackTrace();
				}
    		}
    		op2 = p2;
    		
    		return true;
    	}
    	
    	class SkyRenderer implements GLSurfaceView.Renderer {
    		static final int FIX = 0x10000;
    		final int nStars = 1024;
    		final int bands = 16;
    		
    		long startTick;
    		int w;
    		int h;
    		
    		IntBuffer starPositions;
    		ByteBuffer starColors;
    		
    		IntBuffer hazePositions;
    		ByteBuffer hazeColors;
    		ByteBuffer hazeIndices;
    		
    		IntBuffer terrainPositions;
    		IntBuffer terrainLinerPositions;
    		ByteBuffer terrainIndices;
    		
    		IntBuffer trailPositions;
    		
    		IntBuffer plusPositions;
    		
    		IntBuffer makeFixedBuffer(int len) {
    			return ByteBuffer.allocateDirect(4*len).order(ByteOrder.nativeOrder()).asIntBuffer();
    		}
    		IntBuffer makeFixedBuffer(float [] src) {
    			IntBuffer buffer = makeFixedBuffer(src.length);
    			for(int i = 0; i < src.length; i++) {
    				buffer.put(i, (int)(0x10000*src[i]));
    			}
    			buffer.rewind();
    			return buffer;
    		}
    		
    		ByteBuffer makeByteBuffer(int len) {
    			return ByteBuffer.allocateDirect(len).order(ByteOrder.nativeOrder());
    		}
    		ByteBuffer makeByteBuffer(byte [] src) {
    			ByteBuffer buffer = makeByteBuffer(src.length);
    			buffer.put(src);
    			buffer.rewind();
    			return buffer;
    		}
    		
    		@Override
    		public void onDrawFrame(GL10 gl) {
    			
    			gl.glClear(GL10.GL_COLOR_BUFFER_BIT);
    			
    			gl.glEnableClientState(GL10.GL_VERTEX_ARRAY);
    			gl.glEnableClientState(GL10.GL_COLOR_ARRAY);
    			
    			gl.glVertexPointer(2, GL10.GL_FIXED, 0, hazePositions);
    			gl.glColorPointer(4, GL10.GL_UNSIGNED_BYTE, 0, hazeColors);
    			gl.glDrawElements(GL10.GL_TRIANGLES, hazeIndices.capacity(), GL10.GL_UNSIGNED_BYTE, hazeIndices);
    			

    			gl.glVertexPointer(2, GL10.GL_FIXED, 0, starPositions);
    			gl.glEnable(GL10.GL_BLEND);
    			gl.glBlendFunc(GL10.GL_SRC_ALPHA, GL10.GL_ONE);
    			gl.glPointSize(2f);
    			gl.glEnable(GL10.GL_POINT_SMOOTH);
    			gl.glColorPointer(4, GL10.GL_UNSIGNED_BYTE, 0, starColors);
    			gl.glPushMatrix();
    			gl.glTranslatef(0.5f, 0.5f, 0);
    			if(w > h) {
    				gl.glScalef(1, (float)w/h, 1);
    			} else {
    				gl.glScalef((float)h/w, 1, 1);
    			}
    			gl.glRotatef((SystemClock.uptimeMillis()-startTick)/2500.0f, 0f,0f,1f);
    			gl.glTranslatef(-0.5f, -0.5f, 0);
    			gl.glDrawArrays(GL10.GL_POINTS, 0, starPositions.capacity()/2);
    			gl.glPopMatrix();

    			gl.glDisableClientState(GL10.GL_COLOR_ARRAY);

    			gl.glVertexPointer(2, GL10.GL_FIXED, 0, terrainPositions);
    			gl.glBlendFunc(GL10.GL_SRC_ALPHA, GL10.GL_ONE_MINUS_SRC_ALPHA);
    			gl.glEnable(GL10.GL_LINE_SMOOTH);
    			gl.glColor4f(0f,0f,0f, 1f);
    			gl.glDrawElements(GL10.GL_TRIANGLES, terrainIndices.capacity(), GL10.GL_UNSIGNED_BYTE, terrainIndices);
    			
    			gl.glVertexPointer(2, GL10.GL_FIXED, 0, terrainLinerPositions);
    			gl.glColor4f(0f,0f,0f, 1f);
    			gl.glDrawArrays(GL10.GL_LINE_STRIP, 0, terrainLinerPositions.capacity()/2);
    			
    			gl.glVertexPointer(2, GL10.GL_FIXED, 0, plusPositions);
    			gl.glBlendFunc(GL10.GL_SRC_ALPHA, GL10.GL_ONE);
				gl.glColor4f(1f, 1f, 1f, 0.25f);
				final float tr = 0.1f;
    			if(p1) {
    				gl.glPushMatrix();
    				gl.glTranslatef(x1, y1, 0f);
    				if(h>w) {
    					gl.glScalef(tr*h/w, tr, 1f);
    				} else {
    					gl.glScalef(tr, tr*w/h, 1f);
    				}
    				gl.glDrawArrays(GL10.GL_TRIANGLE_FAN, 0, plusPositions.capacity()/2);
    				gl.glPopMatrix();
    			}
    			
    			if(p2) {
    				gl.glPushMatrix();
    				gl.glTranslatef(x2, y2, 0f);
    				if(h>w) {
    					gl.glScalef(tr*h/w, tr, 1f);
    				} else {
    					gl.glScalef(tr, tr*w/h, 1f);
    				}
    				gl.glDrawArrays(GL10.GL_TRIANGLE_FAN, 0, plusPositions.capacity()/2);
    				gl.glPopMatrix();
    			}
    			
    			gl.glDisable(GL10.GL_BLEND);
    			
    		}

    		@Override
    		public void onSurfaceChanged(GL10 gl, int width, int height) {
    			w = width;
    			h = height;
    			startTick = SystemClock.uptimeMillis();
    			
    			final float[] starPositionsA = new float[2*nStars];
    			final byte[] starColorsA = new byte[4*nStars];
    			for(int i = 0; i < nStars; i++) {
    				float s = (float)Math.random()-0.5f;
    				float t = (float)Math.random()-0.34f;
    				float tt = (float)Math.pow(t*2,3)+0.05f*t + 0.5f*s*s;
    				starPositionsA[2*i+0] = tt+0.34f;
    				starPositionsA[2*i+1] = 1.15f*s+0.5f;
    				
    				byte intensity = (byte)(255*Math.pow(Math.random(),5));
    				starColorsA[4*i+0] = intensity;
    				starColorsA[4*i+1] = intensity;
    				starColorsA[4*i+2] = intensity;
    				starColorsA[4*i+3] = (byte)0xFF;
    			}
    			starPositions = makeFixedBuffer(starPositionsA);
    			starColors = makeByteBuffer(starColorsA);
    			
    			final float[] hazePositionsA = new float[2*2*(bands+1)];
    			final byte[] hazeColorsA = new byte[2*4*(bands+1)];
    			float rStart = 200+100*((float)Math.random()-0.5f);
    			float gStart = 120+30*((float)Math.random()-0.5f);
    			float bStart = 100+40*((float)Math.random()-0.5f);
    			for(int b = 0; b < bands+1; b++) {
    				float t = (float)b/bands;
    				float y = (float)Math.pow(1-t,2);
    				hazePositionsA[4*b+0] = 0;
    				hazePositionsA[4*b+1] = y;
    				hazePositionsA[4*b+2] = 1;
    				hazePositionsA[4*b+3] = y;
    				byte red   = (byte)(rStart*t*t*t);
    				byte green = (byte)(gStart*t*t);
    				byte blue  = (byte)(bStart*t);
    				hazeColorsA[8*b+0] = red;
    				hazeColorsA[8*b+1] = green;
    				hazeColorsA[8*b+2] = blue;
    				hazeColorsA[8*b+3] = (byte)0xFF;
    				hazeColorsA[8*b+4] = red;
    				hazeColorsA[8*b+5] = green;
    				hazeColorsA[8*b+6] = blue;
    				hazeColorsA[8*b+7] = (byte)0xFF;
    			}
    			
    			hazePositions = makeFixedBuffer(hazePositionsA);
    			hazeColors = makeByteBuffer(hazeColorsA);
    			
    			byte[] hazeIndicesA = new byte[(bands+1)*6];
    			for(int b = 0; b < bands+1; b++) {
    				hazeIndicesA[6*b+0] = (byte)(2*b+0);
    				hazeIndicesA[6*b+1] = (byte)(2*b+1);
    				hazeIndicesA[6*b+2] = (byte)(2*b+2);
    				hazeIndicesA[6*b+3] = (byte)(2*b+1);
    				hazeIndicesA[6*b+4] = (byte)(2*b+3);
    				hazeIndicesA[6*b+5] = (byte)(2*b+2);
    			}
    			hazeIndices = makeByteBuffer(hazeIndicesA);
    			
    			float[] terrain = new TerrainGen(6).gen();
    			terrainPositions = makeFixedBuffer(terrain);
    			final float[] terrainLinerPositionsA = new float[terrain.length/2 - 2];
    			for(int i = 0; i < terrainLinerPositionsA.length/2; i++) {
    				terrainLinerPositionsA[2*i + 0] = terrain[4*i+0];
    				terrainLinerPositionsA[2*i + 1] = terrain[4*i+1];
    			}
    			terrainLinerPositions = makeFixedBuffer(terrainLinerPositionsA);
    			final int tStrips = terrainPositions.capacity()/4 - 1;
    			byte[] terrainIndicesA = new byte[(tStrips-1)*6];
    			for(int t = 0; t < tStrips-1; t++) {
    				terrainIndicesA[6*t+0] = (byte)(2*t+0);
    				terrainIndicesA[6*t+1] = (byte)(2*t+1);
    				terrainIndicesA[6*t+2] = (byte)(2*t+2);
    				terrainIndicesA[6*t+3] = (byte)(2*t+1);
    				terrainIndicesA[6*t+4] = (byte)(2*t+3);
    				terrainIndicesA[6*t+5] = (byte)(2*t+2);
    			}
    			terrainIndices = makeByteBuffer(terrainIndicesA);
    			
    			final int plusRes = 32;
    			float[] plusPositionsA = new float[2*(2+plusRes)];
    			plusPositionsA[0] = 0;
    			plusPositionsA[1] = 0;
    			for(int i = 0; i < plusRes; i++) {
    				final float t = (float)(2*Math.PI*i/plusRes);
    				plusPositionsA[2*i+2] = (float)Math.sin(t);
    				plusPositionsA[2*i+3] = (float)Math.cos(t);
    			}
    			plusPositionsA[2*plusRes+2] = 0;
    			plusPositionsA[2*plusRes+3] = 1;
    			plusPositions = makeFixedBuffer(plusPositionsA);
    			
    			gl.glViewport(0, 0, width, height);
    			gl.glClearColor(0f,0f,0f,1f);
    			
    			gl.glLoadIdentity();
    			gl.glTranslatef(-1, -1f, 0);
    			gl.glScalef(2f, 2f, 1);
    		}

    		@Override
    		public void onSurfaceCreated(GL10 gl, EGLConfig config) {
    			// pass
    		}
    		
    	}
    }

    class TerrainGen {
    	final int maxDepth;
    	static final float spreadX = 0.75f;
    	static final float spreadY = 0.35f;
    	
    	public TerrainGen(int md) {
    		maxDepth = md;
    	}
    	
    	private void gen(float[] xs,
    							float[] ys,
    							float leftX,
    							float rightX,
    							float leftY,
    							float rightY,
    							int idx,
    							int depth) {
    		if(depth > 0) {
    			float fracX = (1-spreadX)*0.5f + spreadX*(float)Math.random();
    			float midX = (1-fracX)*leftX + fracX*rightX;
    			float midY = (1-fracX)*leftY + fracX*rightY;
    			float midYPrime = midY + spreadY*(float)Math.random()/(1<<depth);
    			gen(xs, ys, leftX, midX, leftY, midYPrime, idx, depth-1);
    			gen(xs, ys, midX, rightX, midYPrime, rightY, idx+(1<<(depth-1)), depth-1);
    		} else {
    			xs[idx] = (leftX+rightX)/2;
    			ys[idx] = (leftY+rightY)/2;
    		}
    	}
    	
    	public float[] gen() {
    		int terms = 1 << maxDepth;
    		float[] xs = new float[terms];
    		float[] ys = new float[terms];
    		gen(xs, ys, 0f, 1f, 0.5f, 0.5f, 0, maxDepth);
    		float[] result = new float[4*terms+4+4];
    		result[0] = 0.0f;
    		result[1] = transfer(0.5f, 0);
    		result[2] = 0.0f;
    		result[3] = 0.0f;
    		for(int i = 0; i < terms; i++) {
    			result[4*i+4 + 0] = xs[i];
    			result[4*i+4 + 1] = transfer(ys[i], (float)(i+0.5)/terms);
    			result[4*i+4 + 2] = xs[i];
    			result[4*i+4 + 3] = 0;
    		}
    		result[4*terms + 0] = 1.0f;
    		result[4*terms + 1] = transfer(0.5f, 1f);
    		result[4*terms + 2] = 1.0f;
    		result[4*terms + 3] = 0.0f;
    		return result;
    	}
    	
    	float transfer(float h, float d) {
    		return 0.2f*(float)Math.pow(h,5) + 0.2f*(d-0.5f)*(d-0.5f);
    	}
    }
}